@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileOutlineController.h 11617 2011-01-01 20:42:14Z livings124 $
+ * $Id: FileOutlineController.h 13162 2012-01-14 17:12:04Z livings124 $
  *
- * Copyright (c) 2008-2011 Transmission authors and contributors
+ * Copyright (c) 2008-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,7 +30,7 @@
 @interface FileOutlineController : NSObject
 {
     Torrent * fTorrent;
-    NSArray * fFileList;
+    NSMutableArray * fFileList;
     
     IBOutlet FileOutlineView * fOutline;
     
@@ -43,7 +43,7 @@
 
 - (void) setFilterText: (NSString *) text;
 
-- (void) reloadData;
+- (void) refresh;
 
 - (void) setCheck: (id) sender;
 - (void) setOnlySelectedCheck: (id) sender;
