@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: InfoPeersViewController.m 10711 2010-05-31 14:06:54Z livings124 $
+ * $Id: InfoPeersViewController.m 13162 2012-01-14 17:12:04Z livings124 $
  *
- * Copyright (c) 2010 Transmission authors and contributors
+ * Copyright (c) 2010-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -138,7 +138,7 @@
     else
         [fWebSeeds removeAllObjects];
     
-    NSUInteger known = 0, connected = 0, tracker = 0, incoming = 0, cache = 0, lpd = 0, pex = 0, dht = 0, ltep = 0,
+    NSUInteger connected = 0, tracker = 0, incoming = 0, cache = 0, lpd = 0, pex = 0, dht = 0, ltep = 0,
                 toUs = 0, fromUs = 0;
     BOOL anyActive = false;
     for (Torrent * torrent in fTorrents)
@@ -146,8 +146,6 @@
         if ([torrent webSeedCount] > 0)
             [fWebSeeds addObjectsFromArray: [torrent webSeeds]];
         
-        known += [torrent totalPeersKnown];
-        
         if ([torrent isActive])
         {
             anyActive = YES;
@@ -177,7 +175,6 @@
     [fWebSeeds sortUsingDescriptors: [fWebSeedTable sortDescriptors]];
     [fWebSeedTable reloadData];
     
-    NSString * knownString = [NSString stringWithFormat: NSLocalizedString(@"%d known", "Inspector -> Peers tab -> peers"), known];
     if (anyActive)
     {
         NSString * connectedText = [NSString stringWithFormat: NSLocalizedString(@"%d Connected", "Inspector -> Peers tab -> peers"),
@@ -185,7 +182,17 @@
         
         if (connected > 0)
         {
-            NSMutableArray * fromComponents = [NSMutableArray arrayWithCapacity: 6];
+            NSMutableArray * upDownComponents = [NSMutableArray arrayWithCapacity: 2];
+            if (toUs > 0)
+                [upDownComponents addObject: [NSString stringWithFormat:
+                                        NSLocalizedString(@"DL from %d", "Inspector -> Peers tab -> peers"), toUs]];
+            if (fromUs > 0)
+                [upDownComponents addObject: [NSString stringWithFormat:
+                                        NSLocalizedString(@"UL to %d", "Inspector -> Peers tab -> peers"), fromUs]];
+            if ([upDownComponents count] > 0)
+                connectedText = [connectedText stringByAppendingFormat: @": %@", [upDownComponents componentsJoinedByString: @", "]];
+            
+            NSMutableArray * fromComponents = [NSMutableArray arrayWithCapacity: 7];
             if (tracker > 0)
                 [fromComponents addObject: [NSString stringWithFormat:
                                         NSLocalizedString(@"%d tracker", "Inspector -> Peers tab -> peers"), tracker]];
@@ -208,33 +215,20 @@
                 [fromComponents addObject: [NSString stringWithFormat:
                                         NSLocalizedString(@"%d LTEP", "Inspector -> Peers tab -> peers"), ltep]];
             
-            NSMutableArray * upDownComponents = [NSMutableArray arrayWithCapacity: 3];
-            if (toUs > 0)
-                [upDownComponents addObject: [NSString stringWithFormat:
-                                        NSLocalizedString(@"DL from %d", "Inspector -> Peers tab -> peers"), toUs]];
-            if (fromUs > 0)
-                [upDownComponents addObject: [NSString stringWithFormat:
-                                        NSLocalizedString(@"UL to %d", "Inspector -> Peers tab -> peers"), fromUs]];
-            [upDownComponents addObject: knownString];
-            
-            connectedText = [connectedText stringByAppendingFormat: @": %@\n%@", [fromComponents componentsJoinedByString: @", "],
-                                [upDownComponents componentsJoinedByString: @", "]];
+            connectedText = [connectedText stringByAppendingFormat: @"\n%@", [fromComponents componentsJoinedByString: @", "]];
         }
-        else
-            connectedText = [connectedText stringByAppendingFormat: @"\n%@", knownString];
         
         [fConnectedPeersField setStringValue: connectedText];
     }
     else
     {
-        NSString * activeString;
+        NSString * notActiveString;
         if ([fTorrents count] == 1)
-            activeString = NSLocalizedString(@"Transfer Not Active", "Inspector -> Peers tab -> peers");
+            notActiveString = NSLocalizedString(@"Transfer Not Active", "Inspector -> Peers tab -> peers");
         else
-            activeString = NSLocalizedString(@"Transfers Not Active", "Inspector -> Peers tab -> peers");
+            notActiveString = NSLocalizedString(@"Transfers Not Active", "Inspector -> Peers tab -> peers");
         
-        NSString * connectedText = [activeString stringByAppendingFormat: @"\n%@", knownString];
-        [fConnectedPeersField setStringValue: connectedText];
+        [fConnectedPeersField setStringValue: notActiveString];
     }
 }
 
@@ -273,7 +267,7 @@
         if ([ident isEqualToString: @"DL From"])
         {
             NSNumber * rate;
-            return (rate = [webSeed objectForKey: @"DL From Rate"]) ? [NSString stringForSpeedAbbrev: [rate floatValue]] : @"";
+            return (rate = [webSeed objectForKey: @"DL From Rate"]) ? [NSString stringForSpeedAbbrev: [rate doubleValue]] : @"";
         }
         else
             return [webSeed objectForKey: @"Address"];
@@ -292,12 +286,12 @@
         else if ([ident isEqualToString: @"UL To"])
         {
             NSNumber * rate;
-            return (rate = [peer objectForKey: @"UL To Rate"]) ? [NSString stringForSpeedAbbrev: [rate floatValue]] : @"";
+            return (rate = [peer objectForKey: @"UL To Rate"]) ? [NSString stringForSpeedAbbrev: [rate doubleValue]] : @"";
         }
         else if ([ident isEqualToString: @"DL From"])
         {
             NSNumber * rate;
-            return (rate = [peer objectForKey: @"DL From Rate"]) ? [NSString stringForSpeedAbbrev: [rate floatValue]] : @"";
+            return (rate = [peer objectForKey: @"DL From Rate"]) ? [NSString stringForSpeedAbbrev: [rate doubleValue]] : @"";
         }
         else
             return [peer objectForKey: @"IP"];
@@ -358,15 +352,21 @@
             [components addObject: [peer objectForKey: @"Name"]];
         
         const CGFloat progress = [[peer objectForKey: @"Progress"] floatValue];
-        NSString * progressString = [NSString localizedStringWithFormat: NSLocalizedString(@"Progress: %.1f%%",
-                                        "Inspector -> Peers tab -> table row tooltip"), tr_truncd(progress * 100.0, 1)];
+        NSString * progressString = [NSString stringWithFormat: NSLocalizedString(@"Progress: %@",
+                                        "Inspector -> Peers tab -> table row tooltip"),
+                                        [NSString percentString: progress longDecimals: NO]];
         if (progress < 1.0 && [[peer objectForKey: @"Seed"] boolValue])
             progressString = [progressString stringByAppendingFormat: @" (%@)", NSLocalizedString(@"Partial Seed",
                                 "Inspector -> Peers tab -> table row tooltip")];
         [components addObject: progressString];
         
+        NSString * protocolString = [[peer objectForKey: @"uTP"] boolValue] ? @"\u00b5TP" : @"TCP";
         if ([[peer objectForKey: @"Encryption"] boolValue])
-            [components addObject: NSLocalizedString(@"Encrypted Connection", "Inspector -> Peers tab -> table row tooltip")];
+            protocolString = [protocolString stringByAppendingFormat: @" (%@)",
+                                NSLocalizedString(@"encrypted", "Inspector -> Peers tab -> table row tooltip")];
+        [components addObject: [NSString stringWithFormat:
+                                NSLocalizedString(@"Protocol: %@", "Inspector -> Peers tab -> table row tooltip"),
+                                protocolString]];
         
         NSString * portString;
         NSInteger port;
@@ -580,11 +580,8 @@
     //sort by IP after primary sort
     if (useSecond)
     {
-        #warning when 10.6-only, replace with sortDescriptorWithKey:ascending:selector:
-        NSSortDescriptor * secondDescriptor = [[NSSortDescriptor alloc] initWithKey: @"IP" ascending: asc
-                                                                        selector: @selector(compareNumeric:)];
+        NSSortDescriptor * secondDescriptor = [NSSortDescriptor sortDescriptorWithKey: @"IP" ascending: asc selector: @selector(compareNumeric:)];
         [descriptors addObject: secondDescriptor];
-        [secondDescriptor release];
     }
     
     return descriptors;
